@@ -9,6 +9,7 @@ Functions for generating a project from a project template.
 """
 from __future__ import unicode_literals
 from collections import OrderedDict
+import fnmatch
 import io
 import json
 import logging
@@ -20,12 +21,58 @@ from jinja2.environment import Environment
 from jinja2.exceptions import TemplateSyntaxError
 from binaryornot.check import is_binary
 
-from .exceptions import NonTemplatedInputDirException, ContextDecodingException
+from .exceptions import (
+    NonTemplatedInputDirException,
+    ContextDecodingException,
+    FailedHookException,
+    OutputDirExistsException
+)
 from .find import find_template
-from .utils import make_sure_path_exists, work_in
+from .utils import make_sure_path_exists, work_in, rmtree
 from .hooks import run_hook
 
 
+def copy_without_render(path, context):
+    """
+    Returns True if `path` matches some pattern in the
+    `_copy_without_render` context setting.
+
+    :param path: A file-system path referring to a file or dir that
+        should be rendered or just copied.
+    :param context: cookiecutter context.
+    """
+    try:
+        for dont_render in context['cookiecutter']['_copy_without_render']:
+            if fnmatch.fnmatch(path, dont_render):
+                return True
+    except KeyError:
+        return False
+
+    return False
+
+
+def apply_overwrites_to_context(context, overwrite_context):
+    """Modify the given context in place based on the overwrite_context."""
+    for variable, overwrite in overwrite_context.items():
+        if variable not in context:
+            # Do not include variables which are not used in the template
+            continue
+
+        context_value = context[variable]
+
+        if isinstance(context_value, list):
+            # We are dealing with a choice variable
+            if overwrite in context_value:
+                # This overwrite is actually valid for the given context
+                # Let's set it as default (by definition first item in list)
+                # see ``cookiecutter.prompt.prompt_choice_for_config``
+                context_value.remove(overwrite)
+                context_value.insert(0, overwrite)
+        else:
+            # Simply overwrite the value for this variable
+            context[variable] = overwrite
+
+
 def generate_context(context_file='cookiecutter.json', default_context=None,
                      extra_context=None):
     """
@@ -61,9 +108,9 @@ def generate_context(context_file='cookiecutter.json', default_context=None,
     # Overwrite context variable defaults with the default context from the
     # user's global config, if available
     if default_context:
-        obj.update(default_context)
+        apply_overwrites_to_context(obj, default_context)
     if extra_context:
-        obj.update(extra_context)
+        apply_overwrites_to_context(obj, extra_context)
 
     logging.debug('Context generated is {0}'.format(context))
     return context
@@ -95,7 +142,13 @@ def generate_file(project_dir, infile, context, env):
 
     # Render the path to the output file (not including the root project dir)
     outfile_tmpl = Template(infile)
+
     outfile = os.path.join(project_dir, outfile_tmpl.render(**context))
+    file_name_is_empty = os.path.isdir(outfile)
+    if file_name_is_empty:
+        logging.debug('The resulting file name is empty: {0}'.format(outfile))
+        return
+
     logging.debug('outfile is {0}'.format(outfile))
 
     # Just copy over binary files. Don't render.
@@ -128,7 +181,8 @@ def generate_file(project_dir, infile, context, env):
     shutil.copymode(infile, outfile)
 
 
-def render_and_create_dir(dirname, context, output_dir):
+def render_and_create_dir(dirname, context, output_dir,
+                          overwrite_if_exists=False):
     """
     Renders the name of a directory, creates the directory, and
     returns its path.
@@ -143,6 +197,18 @@ def render_and_create_dir(dirname, context, output_dir):
     dir_to_create = os.path.normpath(
         os.path.join(output_dir, rendered_dirname)
     )
+
+    output_dir_exists = os.path.exists(dir_to_create)
+
+    if overwrite_if_exists:
+        if output_dir_exists:
+            logging.debug('Output directory {} already exists,'
+                          'overwriting it'.format(dir_to_create))
+    else:
+        if output_dir_exists:
+            msg = 'Error: "{}" directory already exists'.format(dir_to_create)
+            raise OutputDirExistsException(msg)
+
     make_sure_path_exists(dir_to_create)
     return dir_to_create
 
@@ -157,13 +223,30 @@ def ensure_dir_is_templated(dirname):
         raise NonTemplatedInputDirException
 
 
-def generate_files(repo_dir, context=None, output_dir='.'):
+def _run_hook_from_repo_dir(repo_dir, hook_name, project_dir, context):
+    """
+    Run hook from repo directory, cleaning up project directory if hook fails
+    """
+    with work_in(repo_dir):
+        try:
+            run_hook(hook_name, project_dir, context)
+        except FailedHookException:
+            rmtree(project_dir)
+            logging.error("Stopping generation because %s"
+                          " hook script didn't exit sucessfully" % hook_name)
+            raise
+
+
+def generate_files(repo_dir, context=None, output_dir='.',
+                   overwrite_if_exists=False):
     """
     Renders the templates and saves them to files.
 
     :param repo_dir: Project template input directory.
     :param context: Dict for populating the template's variables.
     :param output_dir: Where to output the generated project dir into.
+    :param overwrite_if_exists: Overwrite the contents of the output directory
+        if it exists
     """
 
     template_dir = find_template(repo_dir)
@@ -172,7 +255,10 @@ def generate_files(repo_dir, context=None, output_dir='.'):
 
     unrendered_dir = os.path.split(template_dir)[1]
     ensure_dir_is_templated(unrendered_dir)
-    project_dir = render_and_create_dir(unrendered_dir, context, output_dir)
+    project_dir = render_and_create_dir(unrendered_dir,
+                                        context,
+                                        output_dir,
+                                        overwrite_if_exists)
 
     # We want the Jinja path and the OS paths to match. Consequently, we'll:
     #   + CD to the template folder
@@ -184,25 +270,62 @@ def generate_files(repo_dir, context=None, output_dir='.'):
     project_dir = os.path.abspath(project_dir)
     logging.debug('project_dir is {0}'.format(project_dir))
 
-    # run pre-gen hook from repo_dir
-    with work_in(repo_dir):
-        run_hook('pre_gen_project', project_dir, context)
+    _run_hook_from_repo_dir(repo_dir, 'pre_gen_project', project_dir, context)
 
     with work_in(template_dir):
         env = Environment(keep_trailing_newline=True)
         env.loader = FileSystemLoader('.')
 
         for root, dirs, files in os.walk('.'):
+            # We must separate the two types of dirs into different lists.
+            # The reason is that we don't want ``os.walk`` to go through the
+            # unrendered directories, since they will just be copied.
+            copy_dirs = []
+            render_dirs = []
+
             for d in dirs:
-                unrendered_dir = os.path.join(project_dir,
-                                              os.path.join(root, d))
-                render_and_create_dir(unrendered_dir, context, output_dir)
+                d_ = os.path.normpath(os.path.join(root, d))
+                # We check the full path, because that's how it can be
+                # specified in the ``_copy_without_render`` setting, but
+                # we store just the dir name
+                if copy_without_render(d_, context):
+                    copy_dirs.append(d)
+                else:
+                    render_dirs.append(d)
+
+            for copy_dir in copy_dirs:
+                indir = os.path.normpath(os.path.join(root, copy_dir))
+                outdir = os.path.normpath(os.path.join(project_dir, indir))
+                logging.debug(
+                    'Copying dir {0} to {1} without rendering'
+                    ''.format(indir, outdir)
+                )
+                shutil.copytree(indir, outdir)
+
+            # We mutate ``dirs``, because we only want to go through these dirs
+            # recursively
+            dirs[:] = render_dirs
+            for d in dirs:
+                unrendered_dir = os.path.join(project_dir, root, d)
+                render_and_create_dir(unrendered_dir, context, output_dir,
+                                      overwrite_if_exists)
 
             for f in files:
-                infile = os.path.join(root, f)
+                infile = os.path.normpath(os.path.join(root, f))
+                if copy_without_render(infile, context):
+                    outfile_tmpl = Template(infile)
+                    outfile_rendered = outfile_tmpl.render(**context)
+                    outfile = os.path.join(project_dir, outfile_rendered)
+                    logging.debug(
+                        'Copying file {0} to {1} without rendering'
+                        ''.format(infile, outfile)
+                    )
+                    shutil.copyfile(infile, outfile)
+                    shutil.copymode(infile, outfile)
+                    continue
                 logging.debug('f is {0}'.format(f))
                 generate_file(project_dir, infile, context, env)
 
-    # run post-gen hook from repo_dir
-    with work_in(repo_dir):
-        run_hook('post_gen_project', project_dir, context)
+    _run_hook_from_repo_dir(repo_dir, 'post_gen_project', project_dir, context)
+
+    return project_dir
