@@ -14,11 +14,14 @@ library rather than a script.
 from __future__ import unicode_literals
 import logging
 import os
+import re
 
-from .config import get_user_config
+from .config import get_user_config, USER_CONFIG_PATH
+from .exceptions import InvalidModeException
 from .prompt import prompt_for_config
 from .generate import generate_context, generate_files
 from .vcs import clone
+from .replay import dump, load
 
 logger = logging.getLogger(__name__)
 
@@ -27,6 +30,20 @@ builtin_abbreviations = {
     'bb': 'https://bitbucket.org/{0}',
 }
 
+REPO_REGEX = """
+(
+((git|ssh|https|http):(//)?)    # something like git:// ssh:// etc.
+ |                              # or
+ (\w+@[\w\.]+)                  # something like user@...
+)
+.*
+"""
+
+
+def is_repo_url(value):
+    """Return True if value is a repository URL."""
+    return bool(re.match(REPO_REGEX, value, re.VERBOSE))
+
 
 def expand_abbreviations(template, config_dict):
     """
@@ -52,7 +69,10 @@ def expand_abbreviations(template, config_dict):
     return template
 
 
-def cookiecutter(template, checkout=None, no_input=False, extra_context=None):
+def cookiecutter(
+        template, checkout=None, no_input=False, extra_context=None,
+        replay=False, overwrite_if_exists=False, output_dir='.',
+        config_file=USER_CONFIG_PATH):
     """
     API equivalent to using Cookiecutter at the command line.
 
@@ -62,16 +82,25 @@ def cookiecutter(template, checkout=None, no_input=False, extra_context=None):
     :param no_input: Prompt the user at command line for manual configuration?
     :param extra_context: A dictionary of context that overrides default
         and user configuration.
+    :param: overwrite_if_exists: Overwrite the contents of output directory
+        if it exists
+    :param output_dir: Where to output the generated project dir into.
+    :param config_file: User configuration file path.
     """
+    if replay and ((no_input is not False) or (extra_context is not None)):
+        err_msg = (
+            "You can not use both replay and no_input or extra_context "
+            "at the same time."
+        )
+        raise InvalidModeException(err_msg)
 
     # Get user config from ~/.cookiecutterrc or equivalent
     # If no config file, sensible defaults from config.DEFAULT_CONFIG are used
-    config_dict = get_user_config()
+    config_dict = get_user_config(config_file=config_file)
 
     template = expand_abbreviations(template, config_dict)
 
-    # TODO: find a better way to tell if it's a repo URL
-    if 'git@' in template or 'https://' in template:
+    if is_repo_url(template):
         repo_dir = clone(
             repo_url=template,
             checkout=checkout,
@@ -83,21 +112,30 @@ def cookiecutter(template, checkout=None, no_input=False, extra_context=None):
         # cookiecutters_dir
         repo_dir = template
 
-    context_file = os.path.join(repo_dir, 'cookiecutter.json')
-    logging.debug('context_file is {0}'.format(context_file))
+    template_name = os.path.basename(template)
 
-    context = generate_context(
-        context_file=context_file,
-        default_context=config_dict['default_context'],
-        extra_context=extra_context,
-    )
+    if replay:
+        context = load(template_name)
+    else:
+        context_file = os.path.join(repo_dir, 'cookiecutter.json')
+        logging.debug('context_file is {0}'.format(context_file))
 
-    # prompt the user to manually configure at the command line.
-    # except when 'no-input' flag is set
-    context['cookiecutter'] = prompt_for_config(context, no_input)
+        context = generate_context(
+            context_file=context_file,
+            default_context=config_dict['default_context'],
+            extra_context=extra_context,
+        )
+
+        # prompt the user to manually configure at the command line.
+        # except when 'no-input' flag is set
+        context['cookiecutter'] = prompt_for_config(context, no_input)
+
+        dump(template_name, context)
 
     # Create project from local context and project template.
-    generate_files(
+    return generate_files(
         repo_dir=repo_dir,
-        context=context
+        context=context,
+        overwrite_if_exists=overwrite_if_exists,
+        output_dir=output_dir
     )
