@@ -34,6 +34,12 @@ from cookiecutter import exceptions
 from cookiecutter import utils
 
 
+@pytest.mark.parametrize('invalid_dirname', ['', '{foo}', '{{foo', 'bar}}'])
+def test_ensure_dir_is_templated_raises(invalid_dirname):
+    with pytest.raises(exceptions.NonTemplatedInputDirException):
+        generate.ensure_dir_is_templated(invalid_dirname)
+
+
 @pytest.fixture(scope='function')
 def remove_additional_folders(request):
     """
@@ -143,6 +149,21 @@ def test_generate_files_output_dir():
     assert os.path.isfile('tests/custom_output_dir/inputpizzä/simple.txt')
 
 
+@pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
+def test_return_rendered_project_dir():
+    os.mkdir('tests/custom_output_dir')
+    project_dir = generate.generate_files(
+        context={
+            'cookiecutter': {'food': 'pizzä'}
+        },
+        repo_dir=os.path.abspath('tests/test-generate-files'),
+        output_dir='tests/custom_output_dir'
+    )
+    assert project_dir == os.path.abspath(
+        'tests/custom_output_dir/inputpizzä/'
+    )
+
+
 @pytest.mark.usefixtures('clean_system', 'remove_additional_folders')
 def test_generate_files_permissions():
     """
