@@ -32,6 +32,19 @@ def test_git_clone():
         utils.rmtree('cookiecutter-pypackage')
 
 
+@skipif_no_network
+def test_git_clone_with_trailing_slash():
+    repo_dir = vcs.clone(
+        'https://github.com/audreyr/cookiecutter-pypackage/'
+    )
+
+    assert repo_dir == 'cookiecutter-pypackage'
+    assert os.path.isfile('cookiecutter-pypackage/README.rst')
+
+    if os.path.isdir('cookiecutter-pypackage'):
+        utils.rmtree('cookiecutter-pypackage')
+
+
 @skipif_no_network
 def test_git_clone_checkout():
     repo_dir = vcs.clone(
@@ -92,3 +105,41 @@ def test_vcs_not_installed(monkeypatch):
     )
     with pytest.raises(exceptions.VCSNotInstalled):
         vcs.clone('http://norepotypespecified.com')
+
+
+@pytest.mark.parametrize('repo_url, exp_repo_type, exp_repo_url', [
+    (
+        "git+https://github.com/pytest-dev/cookiecutter-pytest-plugin.git",
+        "git",
+        "https://github.com/pytest-dev/cookiecutter-pytest-plugin.git"
+    ), (
+        "hg+https://bitbucket.org/foo/bar.hg",
+        "hg",
+        "https://bitbucket.org/foo/bar.hg"
+    ), (
+        "https://github.com/pytest-dev/cookiecutter-pytest-plugin.git",
+        "git",
+        "https://github.com/pytest-dev/cookiecutter-pytest-plugin.git"
+    ), (
+        "https://bitbucket.org/foo/bar.hg",
+        "hg",
+        "https://bitbucket.org/foo/bar.hg"
+    )
+])
+def test_identify_known_repo(repo_url, exp_repo_type, exp_repo_url):
+    assert vcs.identify_repo(repo_url) == (exp_repo_type, exp_repo_url)
+
+
+@pytest.fixture(params=[
+    "foo+git",  # uses explicit identifier with 'git' in the wrong place
+    "foo+hg",  # uses explicit identifier with 'hg' in the wrong place
+    "foo+bar",  # uses explicit identifier with neither 'git' nor 'hg'
+    "foobar"  # no identifier but neither 'git' nor 'bitbucket' in url
+])
+def unknown_repo_type_url(request):
+    return request.param
+
+
+def test_identify_raise_on_unknown_repo(unknown_repo_type_url):
+    with pytest.raises(exceptions.UnknownRepoType):
+        vcs.identify_repo(unknown_repo_type_url)
